@@ -59,14 +59,6 @@ knh_BasicBlock_t* new_BasicBlockLABEL(Ctx *ctx)
 	return bb;
 }
 
-#if defined(K_USING_THREADEDCODE)
-#define TADDR   NULL, 0/*counter*/
-#else
-#define TADDR   0/*counter*/
-#endif/*K_USING_THREADEDCODE*/
-
-#define ASMLINE  0
-
 #define KNH_ASM1(T, ...) { \
 		klr_##T##_t op_ = {TADDR, OPCODE_##T, ASMLINE, ## __VA_ARGS__}; \
 		knh_Gamma_asm(ctx, (knh_opline_t*)(&op_)); \
@@ -198,7 +190,7 @@ static int knh_Gamma_asmJMPF(Ctx *ctx, klr_JMPF_t *op)
 			klr_iJEQ_t *opN = (klr_iJEQ_t*)opP;
 			knh_sfpidx_t a = ((klr_iEQ_t*)opP)->a;
 			knh_sfpidx_t b = ((klr_iEQ_t*)opP)->b;
-			opN->jumppc = ((klr_JMP_t*)op)->jumppc;
+			opN->jumppc = (op)->jumppc;
 			opN->a = a; opN->b = b;
 			opP->opcode = OPCODE_iJEQ + ((opP)->opcode - OPCODE_iEQ);
 			return swap;
@@ -207,7 +199,7 @@ static int knh_Gamma_asmJMPF(Ctx *ctx, klr_JMPF_t *op)
 			klr_iJEQn_t *opN = (klr_iJEQn_t*)opP;
 			knh_sfpidx_t a = ((klr_iEQn_t*)opP)->a;
 			knh_int_t n = ((klr_iEQn_t*)opP)->n;
-			opN->jumppc = ((klr_JMP_t*)op)->jumppc;
+			opN->jumppc = (op)->jumppc;
 			opN->a = a; opN->n = n;
 			opP->opcode = OPCODE_iJEQn + ((opP)->opcode - OPCODE_iEQn);
 			return swap;
@@ -216,7 +208,7 @@ static int knh_Gamma_asmJMPF(Ctx *ctx, klr_JMPF_t *op)
 			klr_fJEQ_t *opN = (klr_fJEQ_t*)opP;
 			knh_sfpidx_t a = ((klr_fEQ_t*)opP)->a;
 			knh_sfpidx_t b = ((klr_fEQ_t*)opP)->b;
-			opN->jumppc = ((klr_JMP_t*)op)->jumppc;
+			opN->jumppc = (op)->jumppc;
 			opN->a = a; opN->b = b;
 			opP->opcode = OPCODE_fJEQ + ((opP)->opcode - OPCODE_fEQ);
 			return swap;
@@ -225,7 +217,7 @@ static int knh_Gamma_asmJMPF(Ctx *ctx, klr_JMPF_t *op)
 			klr_fJEQn_t *opN = (klr_fJEQn_t*)opP;
 			knh_sfpidx_t a = ((klr_fEQn_t*)opP)->a;
 			knh_float_t n = ((klr_fEQn_t*)opP)->n;
-			opN->jumppc = ((klr_JMP_t*)op)->jumppc;
+			opN->jumppc = (op)->jumppc;
 			opN->a = a; opN->n = n;
 			opP->opcode = OPCODE_fJEQn + ((opP)->opcode - OPCODE_fEQn);
 			return swap;
@@ -246,7 +238,7 @@ static knh_opcode_t knh_BasicBlock_opcode(knh_BasicBlock_t *bb)
 }
 
 #define BB(bb)   knh_opcode_tochar(knh_BasicBlock_opcode(bb))
-
+#ifdef K_USING_DEBUG
 static void dumpBB(knh_BasicBlock_t *bb, const char *indent)
 {
 	size_t i;
@@ -265,6 +257,7 @@ static void dumpBB(knh_BasicBlock_t *bb, const char *indent)
 		(void)op;
 	}
 }
+#endif
 
 static void knh_BasicBlock_strip0(Ctx *ctx, knh_BasicBlock_t *bb)
 {
@@ -1123,7 +1116,7 @@ static void KNH_ASM_XMOVx(Ctx *ctx, knh_type_t atype, knh_sfx_t ax, knh_type_t b
 		KNH_ASM1(XBMOV, ax, NC_(espidx));
 	}
 	else {
-		DBG_ASSERT(atype == TYPE_Any || IS_Tnumbox(atype));
+		//DBG_ASSERT(atype == TYPE_Any || IS_Tnumbox(atype));
 		if(IS_Tint(btype)) { // Any a = b; // int b;
 			KNH_ASM1(iMOVx, NC_(espidx), bx);
 			KNH_ASM1(TR, OC_(espidx), SFP_(espidx), RIX_(espidx-espidx), CLASS_type(btype), _BOX);
@@ -1318,6 +1311,13 @@ static knh_opcode_t OPCODE_imn(knh_methodn_t mn, int diff)
 	case MN_opLTE: return OPCODE_iLTE+ diff;
 	case MN_opGT:  return OPCODE_iGT+ diff;
 	case MN_opGTE: return OPCODE_iGTE+ diff;
+#ifdef OPCODE_iAND
+	case MN_opLAND :  return OPCODE_iAND  + diff;
+	case MN_opLOR  :  return OPCODE_iOR   + diff;
+	case MN_opLXOR :  return OPCODE_iXOR  + diff;
+	case MN_opLSFT:  return OPCODE_iLSFT  + diff;
+	case MN_opRSFT:  return OPCODE_iRSFT  + diff;
+#endif
 	}
 	return OPCODE_NOP;
 }
@@ -1509,7 +1509,7 @@ static void KNH_ASM_INLINE(Ctx *ctx, int sfpshift, knh_opline_t *code, size_t is
 		if(knh_opcode_hasjump(op->opcode)) {
 			int jmpidx = code - (knh_opline_t*)(op->p[0]);
 			if (jmpidx < 0) jmpidx = -jmpidx;
-			DBG_ASSERT(jmpidx < isize);
+			DBG_ASSERT(jmpidx < (int)isize);
 			bb[i]->jumpNC = bb[jmpidx];
 			DP(bb[jmpidx])->incoming += 1;
 			op->p[0] = NULL;
@@ -1579,11 +1579,13 @@ static knh_Method_t* knh_lookupDynamicMethod(Ctx *ctx, knh_methodn_t mn)
 	return knh_lookupMethod(ctx, CLASS_Tvoid, mn);
 }
 
+#ifdef OPCODE_FASTCALL0
 static knh_bool_t knh_Method_isFASTCALL0(Ctx *ctx, knh_Method_t *mtd)
 {
 	if(knh_Method_isAbstract(mtd) || knh_Method_isKLRCode(mtd)) return 0;
 	return 1;
 }
+#endif
 
 static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sfpidx)
 {
@@ -1623,7 +1625,7 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 				}
 				else {
 					KNH_ASM1(OGETIDXn, OC_(sfpidx), OC_(a), n);
-					KNH_ASM_UNBOX(ctx, sfpidx, reqt);
+					KNH_ASM_UNBOX(ctx, reqt, sfpidx);
 				}
 			}
 			else {
@@ -1634,7 +1636,7 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 				}
 				else {
 					KNH_ASM1(OGETIDX, OC_(sfpidx), OC_(a), NC_(an));
-					KNH_ASM_UNBOX(ctx, sfpidx, reqt);
+					KNH_ASM_UNBOX(ctx, reqt, sfpidx);
 				}
 			}
 			return;
@@ -1655,7 +1657,7 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 				}
 				else {
 					KNH_ASM1(OSETIDXn, OC_(sfpidx), OC_(a), n, OC_(v));
-					KNH_ASM_UNBOX(ctx, sfpidx, reqt);
+					KNH_ASM_UNBOX(ctx, reqt, sfpidx);
 				}
 			}
 			else {
@@ -1666,7 +1668,7 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 				}
 				else {
 					KNH_ASM1(OSETIDX, OC_(sfpidx), OC_(a), NC_(an), OC_(v));
-					KNH_ASM_UNBOX(ctx, sfpidx, reqt);
+					KNH_ASM_UNBOX(ctx, reqt, sfpidx);
 				}
 			}
 			return;
@@ -1707,6 +1709,20 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 			return;
 		}
 	}
+#endif
+#ifdef OPCODE_bNUL
+	if(mtd_cid == CLASS_Object) {
+		if(mtd_mn == MN_isNull) {
+			int a = TERMs_put(ctx, stmt, 1, cid, local + 1);
+			KNH_ASM1(bNUL, NC_(sfpidx), OC_(a));
+			return;
+		}
+		else if(mtd_mn == MN_isNotNull) {
+			int a = TERMs_put(ctx, stmt, 1, cid, local + 1);
+			KNH_ASM1(bNN, NC_(sfpidx), OC_(a));
+			return;
+		}
+	}
 #endif
 	{
 		knh_index_t deltaidx = knh_Method_indexOfGetterField(mtd);
@@ -1763,6 +1779,7 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 	}
 	L_USECALL:;
 	{
+#ifdef OPCODE_FASTCALL0
 		knh_type_t rtype = knh_type_tocid(ctx, knh_ParamArray_rtype(DP(mtd)->mp), cid);
 		if(DP(stmt)->size == 2 && knh_Method_isFASTCALL0(ctx, mtd)) {
 			int a = local;
@@ -1770,7 +1787,7 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 				a = TERMs_put(ctx, stmt, 1, TYPE_cid(mtd_cid), local + 1);
 			}
 			KNH_P("FASTCALL %s.%s", CLASS__(DP(mtd)->cid), MN__(DP(mtd)->mn));
-			KNH_ASM1(FASTCALL0, RTNIDX_(ctx, sfpidx, rtype), SFP_(a), RIX_(sfpidx - a), SP(mtd)->fcall_1);
+			KNH_ASM1(FASTCALL0, RTNIDX_(ctx, sfpidx, rtype), SFP_(a), RIX_(sfpidx - a), SP(mtd)->fcall_1, mtd);
 			return;
 		}
 		if(DP(stmt)->size == 3 && knh_Method_isStatic(mtd) && knh_Method_isFASTCALL0(ctx, mtd)) {
@@ -1778,9 +1795,10 @@ static void knh_StmtCALL_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sf
 			knh_type_t ptype = knh_type_tocid(ctx, p->type, mtd_cid);
 			int a = TERMs_put(ctx, stmt, 2, ptype, local + 2);
 			KNH_P("STATIC FASTCALL %s.%s", CLASS__(DP(mtd)->cid), MN__(DP(mtd)->mn));
-			KNH_ASM1(FASTCALL0, RTNIDX_(ctx, sfpidx, rtype), SFP_(a - 1), RIX_(sfpidx - (a - 1)), SP(mtd)->fcall_1);
+			KNH_ASM1(FASTCALL0, RTNIDX_(ctx, sfpidx, rtype), SFP_(a - 1), RIX_(sfpidx - (a - 1)), SP(mtd)->fcall_1, mtd);
 			return;
 		}
+#endif
 		if(knh_StmtPARAMs_asm(ctx, stmt, 1, local, cid, mtd)) {
 			knh_type_t rtype = knh_type_tocid(ctx, knh_ParamArray_rtype(DP(mtd)->mp), cid);
 			KNH_ASM_CALL(ctx, reqt, local, rtype, mtd, knh_Method_isStatic(mtd), DP(stmt)->size - 2);
@@ -1875,8 +1893,7 @@ static void knh_StmtTCAST_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int s
 				KNH_ASM1(TR, OC_(local), SFP_(local), RIX_(local-local), DP(tkC)->cid, _CHKTYPE);
 			}
 			else {
-				TODO(); DBG_ABORT();
-				//KNH_ASM(ACAST, local, KNH_NULVAL(CLASS_Translator));
+				KNH_ASM1(ACAST, RTNIDX_(ctx, local, stmt->type), SFP_(local), reqt, (knh_Translator_t*)KNH_NULVAL(CLASS_Translator));
 			}
 		}
 		KNH_ASM_MOVL(ctx, reqt, sfpidx, SP(stmt)->type, local);
@@ -1977,7 +1994,7 @@ static void knh_StmtLET_asm(Ctx *ctx, knh_Stmt_t *stmt, knh_type_t reqt, int sfp
 	}
 }
 
-static knh_Method_t* knh_NameSpace_getFormatterNULL(Ctx *ctx, knh_NameSpace_t *ns, knh_class_t cid, knh_methodn_t mn)
+knh_Method_t* knh_NameSpace_getFormatterNULL(Ctx *ctx, knh_NameSpace_t *ns, knh_class_t cid, knh_methodn_t mn)
 {
 	L_TAIL:;
 	if(DP(ns)->formattersNULL != NULL) {
@@ -2802,7 +2819,7 @@ void knh_Method_asm(Ctx *ctx, knh_Method_t *mtd, knh_Stmt_t *stmtP, knh_Stmt_t *
 //					knh_BasicBlock_t*  lb = new_BasicBlockLABEL(ctx);
 //					KNH_ASM(JMPNN, TADDRx lb, i);
 //					KNH_ASM(OSET, (i), value);
-//					KNH_ASM_UNBOX(ctx, i, knh_Object_cid(value));
+//					KNH_ASM_UNBOX(ctx, knh_Object_cid(value), i);
 //					KNH_ASM_LABEL(ctx, lb);
 //				}
 			}
