@@ -1,20 +1,20 @@
 /****************************************************************************
- * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER  
+ * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  * 
  * Copyright (c) 2006-2010, Kimio Kuramitsu <kimio at ynu.ac.jp>
- *           (c) 2008-      Konoha Team konohaken@googlegroups.com  
+ *           (c) 2008-      Konoha Team konohaken@googlegroups.com
  * All rights reserved.
  * 
- * You may choose one of the following two licenses when you use konoha. 
+ * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  * 
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with K_UNDER_LGPL)
  * (2) Konoha Software Foundation License 1.0
  * 
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
  * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -22,11 +22,11 @@
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *  
+ * 
  ****************************************************************************/
 
 #include"commons.h"
-#include"../../include/konoha/konoha_vm.h"
+#include"../../include/konoha07/konoha_vm.h"
 
 
 #ifdef __cplusplus 
@@ -81,11 +81,11 @@ static const knh_OPDATA_t OPDATA[] = {
 	{"SCALL", 4, { VMT_R, VMT_SFPIDX, VMT_SFPIDX, VMT_MTD, VMT_VOID}}, 
 	{"VCALL", 4, { VMT_R, VMT_SFPIDX, VMT_SFPIDX, VMT_MTD, VMT_VOID}}, 
 	{"VCALL", 4, { VMT_R, VMT_SFPIDX, VMT_SFPIDX, VMT_MTD, VMT_VOID}}, 
-	{"FASTCALL0", 4, { VMT_R, VMT_SFPIDX, VMT_I, VMT_F, VMT_VOID}}, 
+	{"FASTCALL0", 5, { VMT_R, VMT_SFPIDX, VMT_I, VMT_F, VMT_MTD, VMT_VOID}}, 
 	{"RET", 0, { VMT_VOID}}, 
 	{"SCAST", 4, { VMT_R, VMT_SFPIDX, VMT_I, VMT_TRL, VMT_VOID}}, 
 	{"TCAST", 4, { VMT_R, VMT_SFPIDX, VMT_I, VMT_TRL, VMT_VOID}}, 
-	{"ACAST", 4, { VMT_R, VMT_SFPIDX, VMT_I, VMT_TRL, VMT_VOID}}, 
+	{"ACAST", 4, { VMT_R, VMT_SFPIDX, VMT_CID, VMT_TRL, VMT_VOID}}, 
 	{"iCAST", 2, { VMT_RN, VMT_RN, VMT_VOID}}, 
 	{"fCAST", 2, { VMT_RN, VMT_RN, VMT_VOID}}, 
 	{"TR", 5, { VMT_R, VMT_SFPIDX, VMT_I, VMT_CID, VMT_F, VMT_VOID}}, 
@@ -116,6 +116,11 @@ static const knh_OPDATA_t OPDATA[] = {
 	{"iLTE", 3, { VMT_RN, VMT_RN, VMT_RN, VMT_VOID}}, 
 	{"iGT", 3, { VMT_RN, VMT_RN, VMT_RN, VMT_VOID}}, 
 	{"iGTE", 3, { VMT_RN, VMT_RN, VMT_RN, VMT_VOID}}, 
+	{"iAND", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iOR", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iXOR", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iLSFT", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iRSFT", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
 	{"iADDn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
 	{"iSUBn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
 	{"iMULn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
@@ -127,6 +132,11 @@ static const knh_OPDATA_t OPDATA[] = {
 	{"iLTEn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
 	{"iGTn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
 	{"iGTEn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iANDn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iORn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iXORn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iLSFTn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
+	{"iRSFTn", 3+VMTSIZE_int, { VMT_RN, VMT_RN, VMT_INT VMTX_INT, VMT_VOID}}, 
 	{"fNEG", 2, { VMT_RN, VMT_RN, VMT_VOID}}, 
 	{"fADD", 3, { VMT_RN, VMT_RN, VMT_RN, VMT_VOID}}, 
 	{"fSUB", 3, { VMT_RN, VMT_RN, VMT_RN, VMT_VOID}}, 
@@ -196,6 +206,9 @@ static const knh_OPDATA_t OPDATA[] = {
 	{"OSET2", 3, { VMT_RO, VMT_OBJECT, VMT_OBJECT, VMT_VOID}}, 
 	{"OSET3", 4, { VMT_RO, VMT_OBJECT, VMT_OBJECT, VMT_OBJECT, VMT_VOID}}, 
 	{"OSET4", 5, { VMT_RO, VMT_OBJECT, VMT_OBJECT, VMT_OBJECT, VMT_OBJECT, VMT_VOID}}, 
+#ifdef K_USING_SSA
+	{"PHI", 4+VMTSIZE_int, { VMT_R, VMT_R, VMT_R, VMT_INT VMTX_INT, VMT_VOID}}, 
+#endif /* K_USING_SSA */
 	{"NOP", 0, { VMT_VOID}}, 
 };
 
@@ -265,6 +278,11 @@ void knh_opcode_check(void)
 	KNH_ASSERT(sizeof(klr_iLTE_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_iGT_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_iGTE_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iAND_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iOR_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iXOR_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iLSFT_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iRSFT_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_iADDn_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_iSUBn_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_iMULn_t) <= sizeof(knh_opline_t));
@@ -276,6 +294,11 @@ void knh_opcode_check(void)
 	KNH_ASSERT(sizeof(klr_iLTEn_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_iGTn_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_iGTEn_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iANDn_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iORn_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iXORn_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iLSFTn_t) <= sizeof(knh_opline_t));
+	KNH_ASSERT(sizeof(klr_iRSFTn_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_fNEG_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_fADD_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_fSUB_t) <= sizeof(knh_opline_t));
@@ -345,6 +368,9 @@ void knh_opcode_check(void)
 	KNH_ASSERT(sizeof(klr_OSET2_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_OSET3_t) <= sizeof(knh_opline_t));
 	KNH_ASSERT(sizeof(klr_OSET4_t) <= sizeof(knh_opline_t));
+#ifdef K_USING_SSA
+	KNH_ASSERT(sizeof(klr_PHI_t) <= sizeof(knh_opline_t));
+#endif /* K_USING_SSA */
 	KNH_ASSERT(sizeof(klr_NOP_t) <= sizeof(knh_opline_t));
 }
 
@@ -400,11 +426,13 @@ knh_bool_t knh_opcode_hasjump(knh_opcode_t opcode)
 knh_bool_t knh_opcode_usedef(knh_opcode_t opcode, int i)
 {
 	knh_ushort_t type = OPDATA[opcode].types[i];
-	if ((type == VMT_SFPIDX) || (type == VMT_R) ||
-			(type == VMT_RN) || (type == VMT_RO)) {
-		return 1;
-	}
-	return 0;
+	return ((type == VMT_R) || (type == VMT_RN) || (type == VMT_RO));
+}
+/* ------------------------------------------------------------------------ */
+knh_bool_t knh_opcode_sfpidx(knh_opcode_t opcode, int i)
+{
+	knh_ushort_t type = OPDATA[opcode].types[i];
+	return (type == VMT_SFPIDX);
 }
 /* ------------------------------------------------------------------------ */
 void knh_opline_traverse(Ctx *ctx, knh_opline_t *c, knh_Ftraverse ftr)
@@ -412,7 +440,7 @@ void knh_opline_traverse(Ctx *ctx, knh_opline_t *c, knh_Ftraverse ftr)
 	size_t i, size = OPDATA[c->opcode].size;
 	for(i = 0; i < size; i++) {
 		knh_ushort_t vtype = OPDATA[c->opcode].types[i];
-		if(vtype == VMT_OBJECT || vtype == VMT_STRING) {
+		if(vtype == VMT_OBJECT || vtype == VMT_STRING || vtype == VMT_TRL) {
 			KNH_FTR(ctx, ftr, UPCAST(c->p[i]));
 		}
 	}
@@ -421,7 +449,7 @@ void knh_opline_traverse(Ctx *ctx, knh_opline_t *c, knh_Ftraverse ftr)
 
 #ifdef K_USING_RBP
 #define RBP_ASSERT0(N)   	if((N % 2) != 0) {		DBG_P("r=%d", N); 		KNH_ASSERT((N % 2) == 0);	}
-#define RBP_ASSERT1(N)   	if((N % 2) == 0) {		DBG_P("r=%d", N); 		KNH_ASSERT((N % 2) != 0);	}	
+#define RBP_ASSERT1(N)   	if((N % 2) == 0) {		DBG_P("r=%d", N); 		KNH_ASSERT((N % 2) != 0);	}
 #else
 #define RBP_ASSER0(N)
 #define RBP_ASSER1(N)
@@ -496,11 +524,16 @@ void knh_opcode_dump(Ctx *ctx, knh_opline_t *c, knh_OutputStream_t *w, knh_oplin
 			break;
 		}
 		case VMT_INT: {
-			knh_int_t n = ((knh_int_t*)(&(c->p[i])))[0];
-			knh_write_ifmt(ctx, w, K_INT_FMT, n); break;
+			union { knh_int_t i; knh_ndata_t n; } v;
+			v.n = c->data[i];
+			knh_write_ifmt(ctx, w, K_INT_FMT, v.i); break;
+		}
+		case VMT_FLOAT: {
+			union { knh_float_t f; knh_ndata_t n; } v;
+			v.n = c->data[i];
+			knh_float_t f = v.f;
+			knh_write_ffmt(ctx, w, K_FLOAT_FMT, f); break;
 		}
-		case VMT_FLOAT:
-			knh_write_ffmt(ctx, w, K_FLOAT_FMT, *((knh_float_t*)&(c->p[i]))); break;
 		}
 	}
 	knh_write_EOL(ctx, w);
@@ -530,18 +563,25 @@ void knh_opcode_shift(knh_opline_t *c, int shift)
 #define VMCOUNT(op)
 #endif
 
+//#if (defined(__i386__) || defined(__x86_64__)) && (defined(__GNUC__) && __GNUC__ >= 3)
+//#define K_USING_VMASMDISPATCH 1
+//#endif
+
 #ifdef K_USING_THREADEDCODE
 #define CASE(x)   L_##x :
 #define NEXT_OP   (pc->codeaddr)
 #define JUMP      *(NEXT_OP)
-//this is not good when using with FASTCALL0 
-//#if (defined(__i386__) || defined(__x86_64__)) && (defined(__GNUC__) && __GNUC__ >= 3)
-//#define GOTO_NEXT()     asm volatile("jmp *%0;": : "r"(NEXT_OP))
-//#define DISPATCH_INIT() asm("" :: "r"(&vpc))
-//#else
+#ifdef K_USING_VMASMDISPATCH
+#define GOTO_NEXT()     {\
+	void **codeaddr = NEXT_OP;\
+	asm volatile("jmp *%0;": : "g"(codeaddr));\
+	goto *(codeaddr);\
+}
+#define DISPATCH_INIT() asm("" :: "r"(&vpc), "r"(&vshift))
+#else
 #define GOTO_NEXT()     goto *(NEXT_OP)
 #define DISPATCH_INIT()
-//#endif
+#endif
 #define TC(c) 
 #define DISPATCH_START(pc) goto *OPJUMP[pc->opcode]
 #define DISPATCH_END(pc)
@@ -579,27 +619,33 @@ knh_opline_t* knh_VirtualMachine_run(Ctx *ctx, knh_sfp_t *sfp0, knh_opline_t *pc
 		&&L_iNEG, &&L_iADD, &&L_iSUB, &&L_iMUL, 
 		&&L_iDIV, &&L_iMOD, &&L_iEQ, &&L_iNEQ, 
 		&&L_iLT, &&L_iLTE, &&L_iGT, &&L_iGTE, 
-		&&L_iADDn, &&L_iSUBn, &&L_iMULn, &&L_iDIVn, 
-		&&L_iMODn, &&L_iEQn, &&L_iNEQn, &&L_iLTn, 
-		&&L_iLTEn, &&L_iGTn, &&L_iGTEn, &&L_fNEG, 
-		&&L_fADD, &&L_fSUB, &&L_fMUL, &&L_fDIV, 
-		&&L_fEQ, &&L_fNEQ, &&L_fLT, &&L_fLTE, 
-		&&L_fGT, &&L_fGTE, &&L_fADDn, &&L_fSUBn, 
-		&&L_fMULn, &&L_fDIVn, &&L_fEQn, &&L_fNEQn, 
-		&&L_fLTn, &&L_fLTEn, &&L_fGTn, &&L_fGTEn, 
-		&&L_OGETIDX, &&L_OSETIDX, &&L_OGETIDXn, &&L_OSETIDXn, 
-		&&L_NGETIDX, &&L_NSETIDX, &&L_NGETIDXn, &&L_NSETIDXn, 
-		&&L_BGETIDX, &&L_BSETIDX, &&L_BGETIDXn, &&L_BSETIDXn, 
-		&&L_bJNUL, &&L_bJNN, &&L_bJNOT, &&L_iJEQ, 
-		&&L_iJNEQ, &&L_iJLT, &&L_iJLTE, &&L_iJGT, 
-		&&L_iJGTE, &&L_iJEQn, &&L_iJNEQn, &&L_iJLTn, 
-		&&L_iJLTEn, &&L_iJGTn, &&L_iJGTEn, &&L_fJEQ, 
-		&&L_fJNEQ, &&L_fJLT, &&L_fJLTE, &&L_fJGT, 
-		&&L_fJGTE, &&L_fJEQn, &&L_fJNEQn, &&L_fJLTn, 
-		&&L_fJLTEn, &&L_fJGTn, &&L_fJGTEn, &&L_ONMOV, 
-		&&L_OOMOV, &&L_NNMOV, &&L_NSET2, &&L_NSET3, 
-		&&L_NSET4, &&L_OSET2, &&L_OSET3, &&L_OSET4, 
-		&&L_NOP, 
+		&&L_iAND, &&L_iOR, &&L_iXOR, &&L_iLSFT, 
+		&&L_iRSFT, &&L_iADDn, &&L_iSUBn, &&L_iMULn, 
+		&&L_iDIVn, &&L_iMODn, &&L_iEQn, &&L_iNEQn, 
+		&&L_iLTn, &&L_iLTEn, &&L_iGTn, &&L_iGTEn, 
+		&&L_iANDn, &&L_iORn, &&L_iXORn, &&L_iLSFTn, 
+		&&L_iRSFTn, &&L_fNEG, &&L_fADD, &&L_fSUB, 
+		&&L_fMUL, &&L_fDIV, &&L_fEQ, &&L_fNEQ, 
+		&&L_fLT, &&L_fLTE, &&L_fGT, &&L_fGTE, 
+		&&L_fADDn, &&L_fSUBn, &&L_fMULn, &&L_fDIVn, 
+		&&L_fEQn, &&L_fNEQn, &&L_fLTn, &&L_fLTEn, 
+		&&L_fGTn, &&L_fGTEn, &&L_OGETIDX, &&L_OSETIDX, 
+		&&L_OGETIDXn, &&L_OSETIDXn, &&L_NGETIDX, &&L_NSETIDX, 
+		&&L_NGETIDXn, &&L_NSETIDXn, &&L_BGETIDX, &&L_BSETIDX, 
+		&&L_BGETIDXn, &&L_BSETIDXn, &&L_bJNUL, &&L_bJNN, 
+		&&L_bJNOT, &&L_iJEQ, &&L_iJNEQ, &&L_iJLT, 
+		&&L_iJLTE, &&L_iJGT, &&L_iJGTE, &&L_iJEQn, 
+		&&L_iJNEQn, &&L_iJLTn, &&L_iJLTEn, &&L_iJGTn, 
+		&&L_iJGTEn, &&L_fJEQ, &&L_fJNEQ, &&L_fJLT, 
+		&&L_fJLTE, &&L_fJGT, &&L_fJGTE, &&L_fJEQn, 
+		&&L_fJNEQn, &&L_fJLTn, &&L_fJLTEn, &&L_fJGTn, 
+		&&L_fJGTEn, &&L_ONMOV, &&L_OOMOV, &&L_NNMOV, 
+		&&L_NSET2, &&L_NSET3, &&L_NSET4, &&L_OSET2, 
+		&&L_OSET3, &&L_OSET4, 
+#ifdef K_USING_SSA
+		&&L_PHI, 
+#endif /* K_USING_SSA */
+&&L_NOP, 
 	};
 #endif
 	knh_rbp_t *rbp = (knh_rbp_t*)sfp0;
@@ -784,7 +830,7 @@ knh_opline_t* knh_VirtualMachine_run(Ctx *ctx, knh_sfp_t *sfp0, knh_opline_t *pc
 	} 
 	CASE(FASTCALL0) {
 		klr_FASTCALL0_t *op = (klr_FASTCALL0_t*)pc; (void)op; VMCOUNT(pc); 
-		KLR2_FASTCALL0(ctx, op->a, op->thisidx, op->rix, op->fcall);
+		KLR2_FASTCALL0(ctx, op->a, op->thisidx, op->rix, op->fcall, op->mtdNC);
 		pc++;
 		GOTO_NEXT();
 	} 
@@ -808,7 +854,7 @@ knh_opline_t* knh_VirtualMachine_run(Ctx *ctx, knh_sfp_t *sfp0, knh_opline_t *pc
 	} 
 	CASE(ACAST) {
 		klr_ACAST_t *op = (klr_ACAST_t*)pc; (void)op; VMCOUNT(pc); 
-		KLR_ACAST(ctx, op->a, op->b, op->rix, op->cast);
+		KLR_ACAST(ctx, op->a, op->b, op->cid, op->cast);
 		pc++;
 		GOTO_NEXT();
 	} 
@@ -992,6 +1038,36 @@ knh_opline_t* knh_VirtualMachine_run(Ctx *ctx, knh_sfp_t *sfp0, knh_opline_t *pc
 		pc++;
 		GOTO_NEXT();
 	} 
+	CASE(iAND) {
+		klr_iAND_t *op = (klr_iAND_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iAND(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iOR) {
+		klr_iOR_t *op = (klr_iOR_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iOR(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iXOR) {
+		klr_iXOR_t *op = (klr_iXOR_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iXOR(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iLSFT) {
+		klr_iLSFT_t *op = (klr_iLSFT_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iLSFT(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iRSFT) {
+		klr_iRSFT_t *op = (klr_iRSFT_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iRSFT(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
 	CASE(iADDn) {
 		klr_iADDn_t *op = (klr_iADDn_t*)pc; (void)op; VMCOUNT(pc); 
 		KLR_iADDn(ctx, op->c, op->a, op->n);
@@ -1058,6 +1134,36 @@ knh_opline_t* knh_VirtualMachine_run(Ctx *ctx, knh_sfp_t *sfp0, knh_opline_t *pc
 		pc++;
 		GOTO_NEXT();
 	} 
+	CASE(iANDn) {
+		klr_iANDn_t *op = (klr_iANDn_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iANDn(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iORn) {
+		klr_iORn_t *op = (klr_iORn_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iORn(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iXORn) {
+		klr_iXORn_t *op = (klr_iXORn_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iXORn(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iLSFTn) {
+		klr_iLSFTn_t *op = (klr_iLSFTn_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iLSFTn(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
+	CASE(iRSFTn) {
+		klr_iRSFTn_t *op = (klr_iRSFTn_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_iRSFTn(ctx, op->c, op->a, op->n);
+		pc++;
+		GOTO_NEXT();
+	} 
 	CASE(fNEG) {
 		klr_fNEG_t *op = (klr_fNEG_t*)pc; (void)op; VMCOUNT(pc); 
 		KLR_fNEG(ctx, op->c, op->a);
@@ -1472,6 +1578,14 @@ knh_opline_t* knh_VirtualMachine_run(Ctx *ctx, knh_sfp_t *sfp0, knh_opline_t *pc
 		pc++;
 		GOTO_NEXT();
 	} 
+#ifdef K_USING_SSA
+	CASE(PHI) {
+		klr_PHI_t *op = (klr_PHI_t*)pc; (void)op; VMCOUNT(pc); 
+		KLR_PHI(ctx, op->a, op->b, op->c, op->type);
+		pc++;
+		GOTO_NEXT();
+	} 
+#endif /* K_USING_SSA */
 	CASE(NOP) {
 		klr_NOP_t *op = (klr_NOP_t*)pc; (void)op; VMCOUNT(pc); 
 		KLR_NOP(ctx);
