@@ -671,16 +671,18 @@ static void knh_cwb_writeSCRIPT(Ctx *ctx, knh_cwb_t *cwb, knh_bytes_t path)
 			knh_cwb_putc(ctx, cwb, '/');
 		}
 	}
-	{
-		size_t i;
-		for(i = 0; i < path.len; i++) {
-			if(path.ustr[i] == '.' && path.ustr[i+1] == '.'
-				&& (path.ustr[i+2] == '/' || path.ustr[i+2] == '\\')) {
-				i += 2; continue;
-			}
-			knh_Bytes_putc(ctx, cwb->ba, path.ustr[i]);
-		}
-	}
+	//{
+	//	size_t i;
+	//	for(i = 0; i < path.len; i++) {
+	//		if(path.ustr[i] == '.' && path.ustr[i+1] == '.'
+	//			&& (path.ustr[i+2] == '/' || path.ustr[i+2] == '\\')) {
+	//			i += 2;
+	//			continue;
+	//		}
+	//		knh_Bytes_putc(ctx, cwb->ba, path.ustr[i]);
+	//	}
+	//}
+	knh_Bytes_write(ctx, cwb->ba, path);
 	knh_cwb_ospath(ctx, cwb);
 }
 
