@@ -63,7 +63,7 @@
 #define USE_getTimeMilliSecond 1
 
 #include"commons.h"
-#include"../../../include/konoha/konoha_code_.h"
+#include"../../../include/konoha07/konoha_code_.h"
 
 /* ************************************************************************ */
 
@@ -1245,11 +1245,12 @@ static int knh_strregex_regexec(Ctx *ctx, knh_regex_t *reg, const char *str, siz
 		p[e].rm_name.len = 0;
 		e++;
 	}
-	DBG_ASSERT(e < nmatch);
-	p[e].rm_so = -1;
-	p[e].rm_eo = -1;
-	p[e].rm_name.ubuf = NULL;
-	p[e].rm_name.len = 0;
+	if (e < nmatch) {
+		p[e].rm_so = -1;
+		p[e].rm_eo = -1;
+		p[e].rm_name.ubuf = NULL;
+		p[e].rm_name.len = 0;
+	}
 	return (po) ? 0 : -1;
 }
 
@@ -2139,6 +2140,8 @@ knh_Term_t *knh_Stmt_done(Ctx *ctx, knh_Stmt_t *o)
 
 static void knh_Gamma_init(Ctx *ctx, Object *o)
 {
+	knh_Gamma_t *gma = (knh_Gamma_t *) o;
+	gma->uri = 0;
 	knh_intptr_t i;
 	knh_GammaEX_t *b = KNH_MALLOCBODY(ctx, Gamma);
 	knh_bzero(b, sizeof(knh_GammaEX_t));
@@ -2180,13 +2183,13 @@ static void knh_Gamma_traverse(Ctx *ctx, Object *o, knh_Ftraverse ftr)
 		KNH_NULLFTR(ctx, ftr, b->gf[i].value);
 	}
 	KNH_FTR(ctx, ftr, (b->ns));
+	KNH_FTR(ctx, ftr, (b->script));
 	KNH_FTR(ctx, ftr, (b->mtd));
 	KNH_FTR(ctx, ftr, (b->lstacks));
 	KNH_FTR(ctx, ftr, (b->insts));
 	KNH_FTR(ctx, ftr, (b->finallyStmt));
 	KNH_NULLFTR(ctx, ftr, b->symbolDictMap);
 	KNH_NULLFTR(ctx, ftr, b->constPools);
-	KNH_NULLFTR(ctx, ftr, (b->script));
 }
 
 static void knh_Gamma_free(Ctx *ctx, Object *o)
