@@ -466,6 +466,11 @@ static void knh_addConverterDSPI(Ctx *ctx, const char *scheme, const knh_ConvDSP
 	knh_addDSPI(ctx, scheme, (knh_DSPI_t*)d);
 }
 
+static void knh_addMapDSPI(Ctx *ctx, const char *scheme, const knh_MapDSPI_t *d, int isOVERRIDE)
+{
+	knh_addDSPI(ctx, scheme, (knh_DSPI_t*)d);
+}
+
 /* ------------------------------------------------------------------------ */
 
 const knh_PackageLoaderAPI_t* knh_getPackageAPI(void)
@@ -477,7 +482,7 @@ const knh_PackageLoaderAPI_t* knh_getPackageAPI(void)
 		knh_setShellSPI,
 		knh_setEbiSPI,
 		knh_addPathDSPI, knh_addStreamDSPI, knh_addQueryDSPI,
-		knh_addConverterDSPI,
+		knh_addConverterDSPI, knh_addMapDSPI
 	};
 	return &exports;
 }
