@@ -52,6 +52,7 @@ extern "C" {
 //	m->map = m->dspi->init(ctx, init, path, NULL);
 //	return m;
 //}
+static knh_Iterator_t *map_toIterator(Ctx *ctx, knh_Map_t *m, enum mapitr_type type, knh_Fitrnext fnexts[]);
 
 #define K_USING_HASHMAP 1
 #ifdef K_USING_HASHMAP
@@ -375,7 +376,7 @@ static size_t hmap_size(Ctx *ctx, knh_map_t* m)
 	return hmap->size;
 }
 
-static ITRNEXT hmap_key_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
+static ITRNEXT hmap_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
 {
 	knh_Iterator_t *it = sfp[0].it;
 	knh_Map_t *m = (knh_Map_t*) DP(it)->source;
@@ -393,6 +394,44 @@ static ITRNEXT hmap_key_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
 			e = e->next;
 		}
 	}
+	DP(it)->pos = pos;
+	ITREND_();
+}
+
+static ITRNEXT hmap_key_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
+{
+	knh_Iterator_t *it = sfp[0].it;
+	knh_Map_t *m = (knh_Map_t*) DP(it)->source;
+	knh_hmap_t *hmap = m->hmap;
+	knh_hentry_t **hentry = knh_map_hentry(hmap);
+	size_t pos;
+	for(pos = DP(it)->pos; pos < hmap->hmax; pos++) {
+		knh_hentry_t *e = hentry[pos];
+		while(e != NULL) {
+			DP(it)->pos = pos+1;
+			ITRNEXT_(e->key);
+			e = e->next;
+		}
+	}
+	DP(it)->pos = pos;
+	ITREND_();
+}
+
+static ITRNEXT hmap_value_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
+{
+	knh_Iterator_t *it = sfp[0].it;
+	knh_Map_t *m = (knh_Map_t*) DP(it)->source;
+	knh_hmap_t *hmap = m->hmap;
+	knh_hentry_t **hentry = knh_map_hentry(hmap);
+	size_t pos;
+	for(pos = DP(it)->pos; pos < hmap->hmax; pos++) {
+		knh_hentry_t *e = hentry[pos];
+		while(e != NULL) {
+			DP(it)->pos = pos+1;
+			ITRNEXT_(e->value);
+			e = e->next;
+		}
+	}
 
 	DP(it)->pos = pos;
 	ITREND_();
@@ -400,10 +439,16 @@ static ITRNEXT hmap_key_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
 
 static knh_bool_t hmap_setIterator(Ctx *ctx, knh_map_t* m, knh_Iterator_t *it)
 {
-	it->fnext_1 = DP(it)->fnext = hmap_key_next;
+	it->fnext_1 = DP(it)->fnext = hmap_next;
 	return 0;
 }
 
+static knh_Iterator_t *hmap_toIterator(Ctx *ctx, knh_Map_t* m, enum mapitr_type type)
+{
+	knh_Fitrnext fnexts[] = {hmap_next, hmap_key_next, hmap_value_next};
+	return map_toIterator(ctx, m, type, fnexts);
+}
+
 static knh_MapDSPI_t* hmap_config(Ctx *ctx, knh_class_t p1, knh_class_t p2);
 
 static knh_MapDSPI_t MAP_StringObject = {
@@ -411,14 +456,14 @@ static knh_MapDSPI_t MAP_StringObject = {
 	hmap_config,
 	hmap_init, hmap_ftr, hmap_free,
 	hmap_get, hmap_set, hmap_remove, hmap_size,
-	hmap_setIterator,
+	hmap_setIterator,hmap_toIterator,
 };
 
 static knh_MapDSPI_t MAP_StringData = {
 	K_DSPI_MAP, "hash",
 	hmap_config, hmap_init, hmap_ftrdata, hmap_free,
 	hmap_getdata, hmap_setdata, hmap_remove, hmap_size,
-	hmap_setIterator,
+	hmap_setIterator,hmap_toIterator,
 };
 
 //static knh_MapDSPI_t MAP_DataObject = {
@@ -600,7 +645,7 @@ static size_t dmap_size(Ctx *ctx, knh_map_t* m)
 	return dmap->size;
 }
 
-static ITRNEXT dmap_key_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
+static ITRNEXT dmap_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
 {
 	knh_Iterator_t *it = sfp[0].it;
 	knh_Map_t *m = (knh_Map_t*) DP(it)->source;
@@ -620,10 +665,46 @@ static ITRNEXT dmap_key_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
 
 static knh_bool_t dmap_setIterator(Ctx *ctx, knh_map_t* m, knh_Iterator_t *it)
 {
-	it->fnext_1 = DP(it)->fnext = dmap_key_next;
+	it->fnext_1 = DP(it)->fnext = dmap_next;
 	return 0;
 }
 
+static ITRNEXT dmap_key_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
+{
+	knh_Iterator_t *it = sfp[0].it;
+	knh_Map_t *m = (knh_Map_t*) DP(it)->source;
+	knh_dmap_t *dmap = knh_map_dmap(m->dmap);
+	knh_dentry_t *dentry = knh_map_dentry(m->dmap);
+	size_t i;
+	for(i = DP(it)->pos; i < dmap->size; i++) {
+		DP(it)->pos = i+1;
+		ITRNEXT_(dentry[i].key);
+	}
+	DP(it)->pos = i;
+	ITREND_();
+}
+
+static ITRNEXT dmap_value_next(Ctx *ctx, knh_sfp_t *sfp, long rtnidx)
+{
+	knh_Iterator_t *it = sfp[0].it;
+	knh_Map_t *m = (knh_Map_t*) DP(it)->source;
+	knh_dmap_t *dmap = knh_map_dmap(m->dmap);
+	knh_dentry_t *dentry = knh_map_dentry(m->dmap);
+	size_t i;
+	for(i = DP(it)->pos; i < dmap->size; i++) {
+		DP(it)->pos = i+1;
+		ITRNEXT_(dentry[i].value);
+	}
+	DP(it)->pos = i;
+	ITREND_();
+}
+
+static knh_Iterator_t *dmap_toIterator(Ctx *ctx, knh_Map_t* m, enum mapitr_type type)
+{
+	knh_Fitrnext fnexts[] = {dmap_next, dmap_key_next, dmap_value_next};
+	return map_toIterator(ctx, m, type, fnexts);
+}
+
 /* ------------------------------------------------------------------------ */
 /* int */
 //
@@ -946,7 +1027,7 @@ static knh_MapDSPI_t DMAP_StringObject = {
 	dmap_config,
 	dmap_init, dmap_ftr, dmap_free,
 	dmap_sget, dmap_sset, dmap_sremove, dmap_size,
-	dmap_setIterator,
+	dmap_setIterator, dmap_toIterator,
 };
 
 static knh_MapDSPI_t DMAP_StringInt = {
@@ -954,7 +1035,7 @@ static knh_MapDSPI_t DMAP_StringInt = {
 	dmap_config,
 	dmap_init, dmap_ftrdata, dmap_free,
 	dmap_sgetint, dmap_ssetint, dmap_sremovedata, dmap_size,
-	dmap_setIterator,
+	dmap_setIterator, dmap_toIterator,
 };
 
 static knh_MapDSPI_t DMAP_StringFloat = {
@@ -962,7 +1043,7 @@ static knh_MapDSPI_t DMAP_StringFloat = {
 	dmap_config,
 	dmap_init, dmap_ftrdata, dmap_free,
 	dmap_sgetfloat, dmap_ssetfloat, dmap_sremovedata, dmap_size,
-	dmap_setIterator,
+	dmap_setIterator, dmap_toIterator,
 };
 
 static knh_MapDSPI_t* dmap_config(Ctx *ctx, knh_class_t p1, knh_class_t p2)
@@ -1406,6 +1487,35 @@ knh_MapDSPI_t *knh_getMapDSPIfromCID(Ctx *ctx, knh_class_t p1, knh_class_t p2)
 	}
 	return &DMAP_StringObject;
 }
+/* ------------------------------------------------------------------------ */
+
+static knh_Iterator_t *map_toIterator(Ctx *ctx, knh_Map_t *m, enum mapitr_type type, knh_Fitrnext fnexts[])
+{
+	knh_Iterator_t *itr;
+	knh_class_t cid = knh_Object_cid(m);
+	knh_class_t ecid; /* Iterator's entry class id */
+	knh_Fitrnext fnext;
+	switch (type) {
+	case MAPITR_KEY:
+		ecid  = knh_class_p1(cid);
+		fnext = fnexts[1];
+		break;
+	case MAPITR_VALUE:
+		ecid = knh_class_p2(cid);
+		fnext = fnexts[2];
+		break;
+	case MAPITR_KEYVALUE:
+	default:
+		// TODO 
+		ecid = CLASS_Tuple;
+		fnext = fnexts[0];
+		break;
+	}
+	itr = new_Iterator(ctx, ecid, UPCAST(m), NULL);
+	itr->fnext_1 = DP(itr)->fnext = fnext;
+	return itr;
+}
+
 /* ------------------------------------------------------------------------ */
 #ifdef __cplusplus
 }
