@@ -718,16 +718,122 @@ typedef struct _kt_status_t {
 	size_t unitsize;
 	kt_unit_t *uhead;
 	kt_unit_t *current;
+	knh_uint_t seek_cur; // added for rewind
 	size_t     lineno;
 	size_t     sumOfPassed;
 	size_t     sumOfFailed;
+	void (*envConv)(char*, size_t);
 } kt_status_t ;
 
+
+// DOS CRLF
+// UNIX(OSX, Linux) LF
+// Mac Classic CR
+
+#define CR 0x0D
+#define LF 0x0A
+
+enum kt_env {
+	KT_ENV_UNKNOWN,
+	KT_ENV_WINDOWS,
+	KT_ENV_UNIX,
+	KT_ENV_MACOS9,
+};
+
+#if defined(KONOHA_ON_WINDOWS) || defined(KONOHA_ON_MINGW) || defined(KONOHA_ON_CYGWIN)
+static void kt_conv_LFtoCRLF(char *buf, size_t len)
+{
+	// code is UNIX, run on WIN
+	// suppose it ends with LF.
+	DBG_ASSERT(buf[len - 1] == LF);
+	DBG_ASSERT(len < KTEST_LINE_MAX - 1);
+	buf[len - 1] = CR;
+	buf[len] = LF;
+	buf[len + 1] = '\0';
+}
+#endif
+
+#if defined(KONOHA_ON_MACOSX) || defined(KONOHA_ON_LINUX)
+static void kt_conv_CRLFtoLF(char *buf, size_t len)
+{
+	// code is WIN, run on UNIX
+	// suppose it ends with CRLF
+	DBG_ASSERT(buf[len - 1] == LF && buf[len - 2] == CR);
+	buf[len - 2] = LF;
+	buf[len - 1] = '\0';
+}
+#endif
+
+static void kt_conv_nop(char *buf, size_t len)
+{
+	// do nothing.
+}
+
+static void detectEnv (kt_status_t *kt, char *line) {
+	size_t linelen = knh_strlen(line);
+	int i = linelen;
+	char ch = 0;
+	knh_bool_t done_flag = 0;
+	enum kt_env code_env = KT_ENV_UNKNOWN;
+	// choose envConv function;
+	void (*conv_table[4])(char*, size_t) = {
+		kt_conv_nop,
+#if defined(KONOHA_ON_MACOSX) || defined(KONOHA_ON_LINUX)
+		kt_conv_CRLFtoLF,
+		kt_conv_nop, 
+		kt_conv_nop 
+#elif defined(KONOHA_ON_WINDOWS) || defined(KONOHA_ON_MINGW) || defined(KONOHA_ON_CYGWIN)
+		kt_conv_nop,
+		kt_conv_LFtoCRLF,
+		kt_conv_nop
+#else
+	/* we should find out MACOS9 */
+		kt_conv_nop,
+		kt_conv_nop,
+		kt_conv_nop
+#endif
+	};
+	
+	if (linelen <= 2) return;
+	while (done_flag == 0 && i > 0) {
+		ch = line[i--];
+		if (ch == LF || ch == CR) {
+			done_flag = 1;
+		}
+	}
+	if(done_flag == 0) return ;
+	if (ch == LF) {
+		// this is DOS or Unix
+		if (i > 1 && line[i] == CR) {
+			// this is WINDOWS
+			code_env = KT_ENV_WINDOWS;
+		} else {
+			// this is Unix
+			code_env = KT_ENV_UNIX;
+		}
+	} else if (ch == CR) {
+		// this is MacOS9
+		code_env = KT_ENV_MACOS9;
+	}
+	//else {
+	//never come here
+	//  }
+	
+	kt->envConv = conv_table[code_env];
+}
+
 static char *kt_fgets(Ctx *ctx, kt_status_t *kt, char *line, int max)
 {
 	char *ret;
+	size_t len = 0;
 	if ((ret = fgets(line, max - 1, kt->in)) != NULL) {
-		line[strlen(line)] = '\0';
+	    len = knh_strlen(line);
+		DBG_ASSERT(len > 0);
+		line[len] = '\0';
+		kt->seek_cur += (knh_uint_t)len;
+		if (kt->envConv != NULL) {
+		  kt->envConv(line, len);
+		}
 	}
 	return ret;
 }
@@ -791,6 +897,14 @@ static void* test_init(Ctx *ctx, const char *msg, const char *filename)
 		kt->lineno = 0;
 		kt->sumOfPassed = 0;
 		kt->sumOfFailed = 0;
+
+		// check env difference
+		char line[KTEST_LINE_MAX] = {0};
+		kt->envConv = NULL;
+		kt_fgets(ctx, kt, line, KTEST_LINE_MAX);
+		fseek(kt->in, 0, SEEK_SET);
+		kt->seek_cur = 0;
+		detectEnv(kt, line);
 		return kt;
 	}
 }
@@ -849,7 +963,8 @@ static knh_bool_t test_readstmt(Ctx *ctx, void *status, knh_cwb_t *cwb, const kn
 	if (ks != NULL && ks->next != NULL) {
 		ks = ks->next;
 		ku->current = ks;
-		if(ks->testBody.len == 1 && ks->testBody.text[0] == '\n') {
+		// TODO : should also consider for windows 
+		if (ks->testBody.len == 1 && ks->testBody.text[0] == LF) {
 			/* assured as passed */
 			kt->sumOfPassed++;
 			ks->isPassed = 1;
@@ -927,10 +1042,13 @@ static knh_bool_t test_readstmt(Ctx *ctx, void *status, knh_cwb_t *cwb, const kn
 		if (IS_T(line_ptr, '#')) {
 			if (isUnitStarted) {
 				if (ku->stmtsize != 0) {
-				// next unit test. we need to rewind fp;
-					// TODO: alternate fseek for windows implementation
-					long len = (long)knh_strlen(line);
-					fseek(kt->in, -(len + 1), SEEK_CUR);
+					// next unit test. we need to rewind fp;
+					// TODO: alternate fseek for windows implementation -> DONE
+					// see what we have till '10/10/31
+					size_t len = knh_strlen(line_ptr);
+					kt->seek_cur -= (len + 1);
+					//					fseek(kt->in, -(len + 1), SEEK_CUR);
+					fseek(kt->in, (long)kt->seek_cur, SEEK_SET);
 					kt->lineno -= 1;
 					break;
 				} else {
@@ -939,7 +1057,6 @@ static knh_bool_t test_readstmt(Ctx *ctx, void *status, knh_cwb_t *cwb, const kn
 			}
 			line_ptr += 4;
 			size_t title_len = knh_strlen(line_ptr);
-
 			kt_truncate_line(line_ptr);
 			if (kt->unitsize == 0) {
 				kt->uhead = new_kt_unit(ctx, line_ptr);
@@ -957,11 +1074,10 @@ static knh_bool_t test_readstmt(Ctx *ctx, void *status, knh_cwb_t *cwb, const kn
 			}
 			continue;
 		}
-
+		
 		if (IS_T(line_ptr, '>')) {
 			line_ptr += 4;
 			if (ku == NULL) continue; // invalid test stmt
-//			kt_truncate_line(line_ptr);
 			if (ku->stmtsize == 0) {
 				ku->shead = new_kt_stmt(ctx, line_ptr);
 				ks = ku->shead;
@@ -1012,10 +1128,10 @@ static knh_bool_t test_readstmt(Ctx *ctx, void *status, knh_cwb_t *cwb, const kn
 		}
 		// handle result
 		if (isResultReady) {
-			if (line_ptr[0] != '\n' && line_ptr[0] != '\0') {
+		  // should also consider CR+LF..
+		  if ((line_ptr[0] != LF) && line_ptr[0] != '\0') {
 				add_kt_stmt_result(ctx, ks, line_ptr);
 			}
-
 		}
 	}
 	if (isUnitStarted && ku->current != NULL) {
@@ -1052,7 +1168,6 @@ static void test_display(Ctx *ctx, void *status, const char* result, const knh_S
 	if (ku == NULL) return;
 	ks = ku->current;
 	size_t len = knh_strlen(result);
-
 	// we cannot know if there is a result or?
 	if (ks->testResult.len == 0) {
 		//ignoring result;
@@ -1106,7 +1221,7 @@ static void test_cleanup(Ctx *ctx, void *status)
 		size_t i, j;
 		/* traverse test result, and sum up */
 		size_t unit_passed = 0, isAllPassed = 1, line = 0;
-		size_t total_stmts = 0, ignore_units = 0; /* for TODOs, and Empties */
+		size_t ignore_units = 0; /* for TODOs, and Empties */
 		for (i = 1; i <= kt->unitsize; i++) {
 			if (ku->stmtsize ==0) {
 				clean_ku = ku;
@@ -1119,7 +1234,6 @@ static void test_cleanup(Ctx *ctx, void *status)
 				KNH_FREE(ctx, clean_ku, sizeof(kt_unit_t));
 				continue;
 			}// ignore
-			total_stmts += ku->stmtsize;
 			for (j = 1; j <= ku->stmtsize; j++) {
 				if (ks->isPassed == 0 /*failed*/) {
 					line = j;
@@ -1162,11 +1276,11 @@ static void test_cleanup(Ctx *ctx, void *status)
 }
 
 static const knh_ShellSPI_t testSPI = {
-		"konoha builtin test mode",
-		test_init,
-		test_readstmt,
-		test_display,
-		test_cleanup,
+	"konoha builtin test mode",
+	test_init,
+	test_readstmt,
+	test_display,
+	test_cleanup,
 };
 
 KNHAPI(void) konoha_runTest(konoha_t konoha, int argc, const char **argv)
