@@ -545,10 +545,14 @@ knh_bool_t knh_write_ndata(Ctx *ctx, knh_OutputStream_t *w, knh_class_t cid, knh
 	case CLASS_Int:
 		knh_write_ifmt(ctx, w, K_INT_FMT, d);
 		return 1;
-	case CLASS_Float:
-		knh_write_ffmt(ctx, w, K_FLOAT_FMT, ((knh_float_t*)&d)[0]);
+	case CLASS_Float: {
+		union { knh_float_t f; knh_ndata_t d; } v;
+		v.d = d;
+		//knh_write_ffmt(ctx, w, K_FLOAT_FMT, ((knh_float_t*)&d)[0]);
+		knh_write_ffmt(ctx, w, K_FLOAT_FMT, v.f);
 		return 1;
 	}
+	}
 	return 0;
 }
 
